@@ -15,7 +15,6 @@ import android.widget.HorizontalScrollView;
 import android.widget.LinearLayout;
 import android.widget.TextView;
 
-import com.dsp_faust.dsp_faust;
 import com.illposed.osc.OSCListener;
 import com.illposed.osc.OSCMessage;
 
@@ -89,10 +88,10 @@ public class UI {
          */
         
         if (parametersInfo.accgyrType[index] == 0) {
-            dsp_faust.setAccConverter(index, -1, 0, 0, 0, 0); // -1 means no mapping
-            dsp_faust.setGyrConverter(index, -1, 0, 0, 0, 0); // -1 means no mapping
+            FaustActivity.dspFaust.setAccConverter(index, -1, 0, 0, 0, 0); // -1 means no mapping
+            FaustActivity.dspFaust.setGyrConverter(index, -1, 0, 0, 0, 0); // -1 means no mapping
         } else if (parametersInfo.accgyrType[index] <= 3) {
-            dsp_faust.setAccConverter(index,
+            FaustActivity.dspFaust.setAccConverter(index,
                                       parametersInfo.accgyrType[index] - 1,  // Java : from 0 to 3 (0 means no mapping), C : -1 to 2 (-1 means no mapping)
                                       parametersInfo.accgyrCurve[index],
                                       parametersInfo.accgyrMin[index],
@@ -100,7 +99,7 @@ public class UI {
                                       parametersInfo.accgyrMax[index]);
             
         } else {
-            dsp_faust.setGyrConverter(index,
+            FaustActivity.dspFaust.setGyrConverter(index,
                                       parametersInfo.accgyrType[index] - 4,  // Java : from 0 to 3 (0 means no mapping), C : -1 to 2 (-1 means no mapping)
                                       parametersInfo.accgyrCurve[index],
                                       parametersInfo.accgyrMin[index],
@@ -117,10 +116,10 @@ public class UI {
 		parametersWindow = new ConfigWindow();
 		
 		// get the JSON description from the native code
-		JSONparameters = dsp_faust.getJSON();
+		JSONparameters = FaustActivity.dspFaust.getJSONUI();
 		
 		// get the total number of parameters
-		int numberOfParameters = dsp_faust.getParamsCount();
+		int numberOfParameters = FaustActivity.dspFaust.getParamsCount();
 		
 		// get the number of each UI elements
 		int numberOfVsliders = countStringOccurrences(JSONparameters,"vslider");
@@ -139,7 +138,7 @@ public class UI {
 		
 		// the addresses of each param in the tree are retrieved from the native code
 		for(int i = 0; i < numberOfParameters; i++){
-			parametersInfo.address[i] = dsp_faust.getParamAddress(i);
+			parametersInfo.address[i] = FaustActivity.dspFaust.getParamAddress(i);
 		}
 		
 		// the saved parameters of each UI element are retrieved
@@ -187,22 +186,22 @@ public class UI {
 		JSONArray uiArray = getJSONui();
 		
 		// TODO Dirty way to detect keyboard: could be improved
-		if (uiArray.toString().contains("\"style\":\"keyboard\"")) {
-			hasKeyboard = true;
-			hasMulti = false;
-		}
-		else if (uiArray.toString().contains("\"style\":\"multi\"")) {
-			hasMulti = true;
-			hasKeyboard = false;
-		}
-		else if (uiArray.toString().contains("\"style\":\"multikeyboard\"")) {
-			hasMulti = true;
-			hasKeyboard = true;
-		}
-		else {
-			hasMulti = false;
-			hasKeyboard = false;
-		}
+        if (uiArray.toString().contains("\"style\":\"keyboard\"")) {
+            hasMulti = false;
+            hasKeyboard = true;
+        }
+        else if (uiArray.toString().contains("\"style\":\"multi\"")) {
+            hasMulti = true;
+            hasKeyboard = false;
+        }
+        else if (uiArray.toString().contains("\"style\":\"multikeyboard\"")) {
+            hasMulti = true;
+            hasKeyboard = true;
+        }
+        else {
+            hasMulti = false;
+            hasKeyboard = false;
+        }
 		
 		parseJSON(c,uiArray,mainGroup,groupLevel,0,Math.round(screenSizeX*(1+parametersInfo.zoom*0.1f)));
 	}
@@ -244,7 +243,8 @@ public class UI {
 	 * 	currentViewWidth: the current group width
 	 */
 	public void parseJSON(Context c, JSONArray uiArray, LinearLayout currentGroup, 
-			int currentGroupLevel, int currentGroupType, int currentViewWidth) {
+                        int currentGroupLevel, int currentGroupType, int currentViewWidth)
+    {
 		int nItemsTopLevel = uiArray.length(), groupDivisions;
 		JSONObject currentObject = new JSONObject();
 		JSONArray currentArray = new JSONArray();
@@ -307,13 +307,13 @@ public class UI {
                 if (metaDataHidden.equals("1")) {
                     paramVisible = false;
                 }
-
-                if (currentObject.getString("type").equals("vgroup")) {
+                // tgroup handled like vgroup
+                if (currentObject.getString("type").equals("tgroup") || currentObject.getString("type").equals("vgroup")) {
 					currentArray = currentObject.getJSONArray("items");
 					vgroup(c,currentArray,currentGroup,currentObject.getString("label"),
 							currentGroupLevel,groupDivisions,currentViewWidth);
 				}
-				else if (currentObject.getString("type").equals("hgroup")){
+            	else if (currentObject.getString("type").equals("hgroup")){
 					currentArray = currentObject.getJSONArray("items");
 					hgroup(c,currentArray,currentGroup,currentObject.getString("label"),
 							currentGroupLevel,groupDivisions,currentViewWidth);
@@ -351,7 +351,7 @@ public class UI {
                     else if (metaDataStyle.contains("radio")) {
 						radio(c,currentGroup,currentObject.getString("address"),
 								currentObject.getString("label"),
-								localScreenWidth,localBackgroundColor,metaDataStyle,0,paramVisible);
+                              localScreenWidth,localBackgroundColor,metaDataStyle,(currentObject.getString("type").equals("vslider")?0:1),paramVisible);
 					}
                     else if (currentObject.getString("type").equals("vslider")) {
 						vslider(c,currentGroup,currentObject.getString("address"),
@@ -414,23 +414,23 @@ public class UI {
 		}
 		for (int i = 0; i < parameterNumber; i++) {
 			if (parametersInfo.parameterType[i] == 0) {       //0: hslider
-				hsliders[parametersCounters[0]].setValue(dsp_faust.getParam(hsliders[parametersCounters[0]].address));
+				hsliders[parametersCounters[0]].setValue(FaustActivity.dspFaust.getParamValue(hsliders[parametersCounters[0]].address));
 				parametersCounters[0]++;
 			}
 			else if (parametersInfo.parameterType[i] == 1) {  //1: vslider
-				vsliders[parametersCounters[1]].setValue(dsp_faust.getParam(vsliders[parametersCounters[1]].address));
+				vsliders[parametersCounters[1]].setValue(FaustActivity.dspFaust.getParamValue(vsliders[parametersCounters[1]].address));
 				parametersCounters[1]++;
 			}
 			else if (parametersInfo.parameterType[i] == 2) {  //2 : knob
-				knobs[parametersCounters[2]].setValue(dsp_faust.getParam(knobs[parametersCounters[2]].address));
+				knobs[parametersCounters[2]].setValue(FaustActivity.dspFaust.getParamValue(knobs[parametersCounters[2]].address));
 				parametersCounters[2]++;
 			}
 			else if (parametersInfo.parameterType[i] == 3) {  //3 : nentry
-				nentries[parametersCounters[3]].setValue(dsp_faust.getParam(nentries[parametersCounters[3]].address));
+				nentries[parametersCounters[3]].setValue(FaustActivity.dspFaust.getParamValue(nentries[parametersCounters[3]].address));
 				parametersCounters[3]++;
 			}
             else if (parametersInfo.parameterType[i] == 9) {  //9 : hbargraph
-                bargraphs[parametersCounters[9]].setValue(dsp_faust.getParam(bargraphs[parametersCounters[9]].address));
+                bargraphs[parametersCounters[9]].setValue(FaustActivity.dspFaust.getParamValue(bargraphs[parametersCounters[9]].address));
                 parametersCounters[9]++;
                 
                 //Log.d("FaustJava", "updateUIstate bargraphs");
@@ -481,7 +481,7 @@ public class UI {
         menus[parametersCounters[4]].linkTo(parametersInfo);
 	    menus[parametersCounters[4]].addTo(currentGroup);
         
-        //dsp_faust.setParam(address, init);
+        //FaustActivity.dspFaust.setParamValue(address, init);
 	    
 	    parametersInfo.parameterType[parameterNumber] = 4;
 	    parametersInfo.localId[parameterNumber] = parametersCounters[4];
@@ -515,7 +515,7 @@ public class UI {
         radios[parametersCounters[5]].linkTo(parametersInfo);
 	    radios[parametersCounters[5]].addTo(currentGroup);
         
-        //dsp_faust.setParam(address, init);
+        //FaustActivity.dspFaust.setParamValue(address, init);
 	    
 	    parametersInfo.parameterType[parameterNumber] = 5;
 	    parametersInfo.localId[parameterNumber] = parametersCounters[5];
@@ -553,7 +553,7 @@ public class UI {
         
         // Accelerometer mappings restored before so that we are sure they are allocated on C side before restoring the actual values...
         updateAccGyr(parametersInfo, parameterNumber);
-        dsp_faust.setParam(address, init);
+        FaustActivity.dspFaust.setParamValue(address, init);
       
         // OSC listener
         final int parameterId = parametersCounters[0];
@@ -607,7 +607,7 @@ public class UI {
         
         // Accelerometer mappings restored before so that we are sure they are allocated on C side before restoring the actual values...
         updateAccGyr(parametersInfo, parameterNumber);
-        dsp_faust.setParam(address, init);
+        FaustActivity.dspFaust.setParamValue(address, init);
 
         // OSC listener
         final int parameterId = parametersCounters[1];
@@ -661,7 +661,7 @@ public class UI {
         
         // Accelerometer mappings restored before so that we are sure they are allocated on C side before restoring the actual values...
         updateAccGyr(parametersInfo, parameterNumber);
-        dsp_faust.setParam(address, init);
+        FaustActivity.dspFaust.setParamValue(address, init);
        
         // OSC listener
         final int parameterId = parametersCounters[2];
@@ -714,7 +714,7 @@ public class UI {
         
         // Accelerometer mappings restored before so that we are sure they are allocated on C side before restoring the actual values...
         updateAccGyr(parametersInfo, parameterNumber);
-        dsp_faust.setParam(address, init);
+        FaustActivity.dspFaust.setParamValue(address, init);
     
         // OSC listener
         final int parameterId = parametersCounters[3];
@@ -782,8 +782,7 @@ public class UI {
 		checkboxes[parametersCounters[7]].setStatus(init);
 	    checkboxes[parametersCounters[7]].linkTo(parametersInfo);
 	    if(visibility) checkboxes[parametersCounters[7]].addTo(currentGroup);
-        
-        dsp_faust.setParam(address, init);
+        FaustActivity.dspFaust.setParamValue(address, init);
 
         // OSC listener
         final int parameterId = parametersCounters[7];
