@@ -41,6 +41,7 @@
 #include <stdio.h>
 #include <vector>
 #include <iostream>
+#include <sys/time.h>
 
 #include <AudioToolbox/AudioConverter.h>
 #include <CoreAudio/CoreAudio.h>
@@ -48,9 +49,7 @@
 #include <CoreServices/CoreServices.h>
 
 #include "faust/audio/audio.h"
-#include "faust/audio/dsp.h"
-
-#include <sys/time.h>
+#include "faust/dsp/dsp.h"
 
 /******************************************************************************
 *******************************************************************************
@@ -284,7 +283,7 @@ class TCoreAudioRenderer
             }  
             */          
             
-            printf("GetDefaultDevice : input = %d output = %d\n", inDefault, outDefault);
+            //printf("GetDefaultDevice : input = %d output = %d\n", inDefault, outDefault);
             
             // Duplex mode
             if (inChan > 0 && outChan > 0) {
@@ -296,7 +295,7 @@ class TCoreAudioRenderer
                     if (CreateAggregateDevice(inDefault, outDefault, sample_rate) != noErr) {
                         return kAudioHardwareBadDeviceError;
                     }
-                    printf("fAggregateDeviceID %d\n", fAggregateDeviceID);
+                    //printf("fAggregateDeviceID %d\n", fAggregateDeviceID);
                     *device = fAggregateDeviceID;
                     goto end;
                 }
@@ -319,7 +318,7 @@ class TCoreAudioRenderer
                 // Otherwise force the one we want...
                 SetupSampleRateAux(*device, sample_rate);
             }
-            printf("samplerate %d\n", sample_rate);
+            //printf("samplerate %d\n", sample_rate);
             fSampleRate = sample_rate;
             return noErr;
         }
@@ -330,19 +329,19 @@ class TCoreAudioRenderer
             AudioObjectID sub_device[32];
             UInt32 outSize = sizeof(sub_device);
             
-            printf("CreateAggregateDevice : input device %d\n", captureDeviceID);
+            //printf("CreateAggregateDevice : input device %d\n", captureDeviceID);
             
             err = AudioDeviceGetProperty(captureDeviceID, 0, kAudioDeviceSectionGlobal, kAudioAggregateDevicePropertyActiveSubDeviceList, &outSize, sub_device);
             std::vector<AudioDeviceID> captureDeviceIDArray;
             
             if (err != noErr) {
-                printf("Input device does not have subdevices\n");
+                //printf("Input device does not have subdevices\n");
                 captureDeviceIDArray.push_back(captureDeviceID);
             } else {
                 int num_devices = outSize / sizeof(AudioObjectID);
-                printf("Input device has %d subdevices\n", num_devices);
+                //printf("Input device has %d subdevices\n", num_devices);
                 for (int i = 0; i < num_devices; i++) {
-                    printf("Input sub_device %d\n", sub_device[i]);
+                    //printf("Input sub_device %d\n", sub_device[i]);
                     captureDeviceIDArray.push_back(sub_device[i]);
                 }
             }
@@ -352,13 +351,13 @@ class TCoreAudioRenderer
             std::vector<AudioDeviceID> playbackDeviceIDArray;
             
             if (err != noErr) {
-                printf("Output device does not have subdevices\n");
+                //printf("Output device does not have subdevices\n");
                 playbackDeviceIDArray.push_back(playbackDeviceID);
             } else {
                 int num_devices = outSize / sizeof(AudioObjectID);
-                printf("Output device has %d subdevices\n", num_devices);
+                //printf("Output device has %d subdevices\n", num_devices);
                 for (int i = 0; i < num_devices; i++) {
-                    printf("Output sub_device %d\n", sub_device[i]);
+                    //printf("Output sub_device %d\n", sub_device[i]);
                     playbackDeviceIDArray.push_back(sub_device[i]);
                 }
             }
@@ -723,10 +722,28 @@ class TCoreAudioRenderer
             return -1;
         }
 
-        OSStatus DestroyAggregateDevice()
+        void DestroyAggregateDevice()
         {   
-            // No more needed : will be done when process quits...
-            return noErr;
+            if (fAggregateDeviceID > 0) {
+                OSStatus osErr = noErr;
+                AudioObjectPropertyAddress pluginAOPA;
+                pluginAOPA.mSelector = kAudioPlugInDestroyAggregateDevice;
+                pluginAOPA.mScope = kAudioObjectPropertyScopeGlobal;
+                pluginAOPA.mElement = kAudioObjectPropertyElementMaster;
+                UInt32 outDataSize;
+                if (fAggregatePluginID > 0)   {
+                    osErr = AudioObjectGetPropertyDataSize(fAggregatePluginID, &pluginAOPA, 0, NULL, &outDataSize);
+                    if (osErr != noErr) {
+                        printf("TCoreAudioRenderer::DestroyAggregateDevice : AudioObjectGetPropertyDataSize error\n");
+                        printError(osErr);
+                    }
+                    osErr = AudioObjectGetPropertyData(fAggregatePluginID, &pluginAOPA, 0, NULL, &outDataSize, &fAggregateDeviceID);
+                    if (osErr != noErr) {
+                        printf("TCoreAudioRenderer::DestroyAggregateDevice : AudioObjectGetPropertyData error\n");
+                        printError(osErr);
+                    }
+                }
+            }
         }
 
         OSStatus GetDeviceNameFromID(AudioDeviceID id, char* name)
@@ -750,7 +767,7 @@ class TCoreAudioRenderer
                 printError(err);
                 return -1;
             } else {
-                printf("SetupBufferSize : buffer size range min = %ld max = %ld\n", (int)buffer_size_range.mMinimum, (int)buffer_size_range.mMaximum);
+                //printf("SetupBufferSize : buffer size range min = %ld max = %ld\n", (int)buffer_size_range.mMinimum, (int)buffer_size_range.mMaximum);
             }
             
             outSize = sizeof(UInt32);
@@ -760,7 +777,7 @@ class TCoreAudioRenderer
                 printError(err);
                 return -1;
             } else {
-                printf("SetupBufferSize : current buffer size %ld\n", current_buffer_size);
+                //printf("SetupBufferSize : current buffer size %ld\n", current_buffer_size);
             }
 
             // If needed, set new buffer size
@@ -809,7 +826,7 @@ class TCoreAudioRenderer
                 // Remove BS change notification
                 AudioDeviceRemovePropertyListener(fDeviceID, 0, true, kAudioDevicePropertyBufferFrameSize, BSNotificationCallback);
             } else {
-                printf("Keep current buffer size = %ld\n", current_buffer_size);
+                //printf("Keep current buffer size = %ld\n", current_buffer_size);
             }
             
             fBufferSize = current_buffer_size;
@@ -833,7 +850,7 @@ class TCoreAudioRenderer
             switch (inPropertyID) {
 
                 case kAudioDevicePropertyBufferFrameSize: {
-                    printf("BSNotificationCallback kAudioDevicePropertyBufferFrameSize\n");
+                    //printf("BSNotificationCallback kAudioDevicePropertyBufferFrameSize\n");
                     // Check new buffer size
                     UInt32 current_buffer_size;
                     UInt32 outSize = sizeof(UInt32);
@@ -842,7 +859,7 @@ class TCoreAudioRenderer
                         printf("Cannot get current buffer size\n");
                         printError(err);
                     } else {
-                        printf("BSNotificationCallback : checked current buffer size = %d\n", current_buffer_size);
+                        //printf("BSNotificationCallback : checked current buffer size = %d\n", current_buffer_size);
                     }
                     driver->fState = true;
                     break;
@@ -956,15 +973,13 @@ class TCoreAudioRenderer
                 for (int i = 0; i < fDevNumOutChans; i++) {
                     fOutChannel[i] = (float*)ioData->mBuffers[i].mData;
                 }
-                fDSP->compute(inNumberFrames, fInChannel, fOutChannel);
+                fDSP->compute(double(AudioConvertHostTimeToNanos(inTimeStamp->mHostTime))/1000., inNumberFrames, fInChannel, fOutChannel);
             } else {
                 printf("AudioUnitRender error... %x\n", fInputData);
                 printError(err);
             }
             return err;
         }
-    
-        
         
     public:
 
@@ -1043,8 +1058,7 @@ class TCoreAudioRenderer
             fDeviceID = 0; 
             fAUHAL = 0;
             fState = false;
-            OpenDefault(inChan, outChan, bufferSize, sampleRate);
-            set_dsp(DSP);
+            return OpenDefault(inChan, outChan, bufferSize, sampleRate);
         }
     
         int OpenDefault(int inChan, int outChan, int buffer_size, int& sample_rate)
@@ -1122,10 +1136,10 @@ class TCoreAudioRenderer
             
             if (inChan > 0) {
                 enableIO = 1;
-                printf("OpenAUHAL : setup AUHAL input on\n");
+                //printf("OpenAUHAL : setup AUHAL input on\n");
             } else {
                 enableIO = 0;
-                printf("OpenAUHAL : setup AUHAL input off\n");
+                //printf("OpenAUHAL : setup AUHAL input off\n");
             }
             
             err = AudioUnitSetProperty(fAUHAL, kAudioOutputUnitProperty_EnableIO, kAudioUnitScope_Input, 1, &enableIO, sizeof(enableIO));
@@ -1137,10 +1151,10 @@ class TCoreAudioRenderer
                 
             if (outChan > 0) {
                 enableIO = 1;
-                printf("OpenAUHAL : setup AUHAL output on\n");
+                //printf("OpenAUHAL : setup AUHAL output on\n");
             } else {
                 enableIO = 0;
-                printf("OpenAUHAL : setup AUHAL output off\n");
+                //printf("OpenAUHAL : setup AUHAL output off\n");
             }
             
             err = AudioUnitSetProperty(fAUHAL, kAudioOutputUnitProperty_EnableIO, kAudioUnitScope_Output, 0, &enableIO, sizeof(enableIO));
@@ -1158,12 +1172,11 @@ class TCoreAudioRenderer
                 printError(err);
                 goto error;
             } else {
-                printf("AudioUnitGetPropertyCurrentDevice = %d\n", currAudioDeviceID);
+                //printf("AudioUnitGetPropertyCurrentDevice = %d\n", currAudioDeviceID);
             }
         
             // Setup up choosen device, in both input and output cases
             err = AudioUnitSetProperty(fAUHAL, kAudioOutputUnitProperty_CurrentDevice, kAudioUnitScope_Global, 0, &fDeviceID, sizeof(AudioDeviceID));
-            printf("fDeviceID %d\n", fDeviceID);
             if (err != noErr) {
                 printf("Error calling AudioUnitSetProperty - kAudioOutputUnitProperty_CurrentDevice\n");
                 printError(err);
@@ -1194,7 +1207,7 @@ class TCoreAudioRenderer
                 //printError(err);
             } else {
                 fPhysicalInputs = (err == noErr) ? outSize / sizeof(SInt32) : 0;
-                printf("fPhysicalInputs = %ld\n", fPhysicalInputs);
+                //printf("fPhysicalInputs = %ld\n", fPhysicalInputs);
             }
                     
             err = AudioUnitGetPropertyInfo(fAUHAL, kAudioOutputUnitProperty_ChannelMap, kAudioUnitScope_Output, 0, &outSize, &isWritable);
@@ -1203,7 +1216,7 @@ class TCoreAudioRenderer
                 //printError(err);
             } else {
                 fPhysicalOutputs = (err == noErr) ? outSize / sizeof(SInt32) : 0;
-                printf("fPhysicalOutputs = %ld\n", fPhysicalOutputs);
+                //printf("fPhysicalOutputs = %ld\n", fPhysicalOutputs);
             }
             
             /*
@@ -1348,7 +1361,7 @@ class TCoreAudioRenderer
                 printf("AudioObjectAddPropertyListener() failed\n");
                 return OPEN_ERR;
             } else {
-                printf("AudioObjectAddPropertyListener() OK\n");
+                //printf("AudioObjectAddPropertyListener() OK\n");
             }
             
             property_address.mSelector = kAudioHardwarePropertyDefaultOutputDevice;
@@ -1356,7 +1369,7 @@ class TCoreAudioRenderer
                 printf("AudioObjectAddPropertyListener() failed\n");
                 return OPEN_ERR;
             } else {
-                printf("AudioObjectAddPropertyListener() OK\n");
+                //printf("AudioObjectAddPropertyListener() OK\n");
             }
              
             return NO_ERR;
@@ -1382,6 +1395,8 @@ class TCoreAudioRenderer
             }
             AudioUnitUninitialize(fAUHAL);
             CloseComponent(fAUHAL);
+            fAUHAL = NULL;
+            
             DestroyAggregateDevice();
             
             delete[] fInChannel;
@@ -1451,46 +1466,48 @@ class TCoreAudioRenderer
 *******************************************************************************/
 class coreaudio : public audio {
 
-    TCoreAudioRenderer fAudioDevice;
-	int fSampleRate, fBufferSize;
+    protected:
+        
+        TCoreAudioRenderer fAudioDevice;
+        int fSampleRate, fBufferSize;
 
- public:
-  
-    coreaudio(int srate, int fpb) : fSampleRate(srate), fBufferSize(fpb) {}
-    coreaudio(int fpb) : fSampleRate(-1), fBufferSize(fpb) {}
-	virtual ~coreaudio() { fAudioDevice.Close(); }
+    public:
+      
+        coreaudio(int srate, int bsize) : fSampleRate(srate), fBufferSize(bsize) {}
+            coreaudio(int bsize) : fSampleRate(-1), fBufferSize(bsize) {}
+        virtual ~coreaudio() { fAudioDevice.Close(); }
 
-	virtual bool init(const char* /*name*/, dsp* DSP) 
-    {
-		if (fAudioDevice.OpenDefault(DSP, DSP->getNumInputs(), DSP->getNumOutputs(), fBufferSize, fSampleRate) < 0) {
-			printf("Cannot open CoreAudio device\n");
-			return false;
-		}
-        fAudioDevice.set_dsp(DSP);
-        // If -1 was given, fSampleRate will be changed by OpenDefault
-        DSP->init(fSampleRate);
-        return true;
-    }
+        virtual bool init(const char* /*name*/, dsp* DSP) 
+        {
+            if (fAudioDevice.OpenDefault(DSP, DSP->getNumInputs(), DSP->getNumOutputs(), fBufferSize, fSampleRate) < 0) {
+                printf("Cannot open CoreAudio device\n");
+                return false;
+            }
+            fAudioDevice.set_dsp(DSP);
+            // If -1 was given, fSampleRate will be changed by OpenDefault
+            DSP->init(fSampleRate);
+            return true;
+        }
 
-	virtual bool start() 
-    {
-		if (fAudioDevice.Start() < 0) {
-			printf("Cannot start CoreAudio device\n");
-			return false;
-		}
-		return true;
-	}
+        virtual bool start() 
+        {
+            if (fAudioDevice.Start() < 0) {
+                printf("Cannot start CoreAudio device\n");
+                return false;
+            }
+            return true;
+        }
 
-	virtual void stop() 
-    {
-		fAudioDevice.Stop();
-	}
-    
-    virtual int get_buffer_size() { return fAudioDevice.GetBufferSize(); }
-    virtual int get_sample_rate() { return fAudioDevice.GetSampleRate(); }
-    
-    virtual int get_num_inputs() { return fAudioDevice.GetNumInputs(); }
-    virtual int get_num_outputs() { return fAudioDevice.GetNumOutputs(); }
+        virtual void stop() 
+        {
+            fAudioDevice.Stop();
+        }
+        
+        virtual int get_buffer_size() { return fAudioDevice.GetBufferSize(); }
+        virtual int get_sample_rate() { return fAudioDevice.GetSampleRate(); }
+        
+        virtual int get_num_inputs() { return fAudioDevice.GetNumInputs(); }
+        virtual int get_num_outputs() { return fAudioDevice.GetNumOutputs(); }
 
 };
 
