@@ -200,7 +200,7 @@ inline int int2pow2 (int x) { int r=0; while ((1<<r)<x) r++; return r; }
 *****************************************************************************/
 
 #include "faust/vst/vstui.h"
-#include "faust/audio/dsp.h"
+#include "faust/dsp/dsp.h"
 
 /********************END ARCHITECTURE SECTION (part 1/2)****************/
 
@@ -511,9 +511,10 @@ bool Faust::getProgramNameIndexed (VstInt32 category, VstInt32 index,
 
 const char* Faust::getMetadata(const char* key, const char* defaultString)
 {
-	Meta meta;
-	mydsp::metadata(&meta);
-	return meta.get(key, defaultString);
+    Meta meta;
+    mydsp tmp_dsp;
+    tmp_dsp.metadata(&meta);
+    return meta.get(key, defaultString);
 } // end of getMetadata
 
 //-----------------------------------------------------------------------------
