@@ -281,7 +281,7 @@ static char* legalFileName(Tree t, int n, char* dst)
 	dst[i] = 0;
 	if (strcmp(dst, "process") != 0) { 
 		// if it is not process add the hex address to make the name unique
-		snprintf(&dst[i], n-i, "-%p", t);
+		snprintf(&dst[i], n-i, "-%p", (void*)t);
 	}
 	return dst;
 }
@@ -530,7 +530,7 @@ static void UserInterfaceDescription(Tree box, string& d)
              << boxpp(step)<< ')';
     }
     else {
-        cerr << "INTERNAL ERROR : unknow user interface element " << endl;
+        cerr << "INTERNAL ERROR : unknown user interface element " << endl;
         exit(0);
     }
     d = fout.str();
