@@ -207,7 +207,11 @@ void Klass::printIncludeFile(ostream& fout)
 
     collectIncludeFile(S);
     for (f = S.begin(); f != S.end(); f++) 	{
-        fout << "#include " << *f << "\n";
+        string inc = *f;
+        // Only print non-empty include (inc is actually quoted)
+        if (inc.size() > 2) { 
+            fout << "#include " << *f << "\n";
+        }
     }
 }
 
@@ -221,14 +225,33 @@ void Klass::printAdditionalCode(ostream& fout)
         fout << "#ifndef FAUSTPOWER" << endl;
         fout << "#define FAUSTPOWER" << endl;
         fout << "#include <cmath>" << endl;
-        fout << "template <int N> inline float faustpower(float x)          { return powf(x,N); } " << endl;
-        fout << "template <int N> inline double faustpower(double x)        { return pow(x,N); }"  << endl;
-        if (gFloatSize == 3) fout << "template <int N> inline long double faustpower(long double x) 	{ return powl(x,N); }"  << endl;
+        
         fout << "template <int N> inline int faustpower(int x)              { return faustpower<N/2>(x) * faustpower<N-N/2>(x); } " << endl;
         fout << "template <> 	 inline int faustpower<0>(int x)            { return 1; }" << endl;
         fout << "template <> 	 inline int faustpower<1>(int x)            { return x; }" << endl;
         fout << "template <> 	 inline int faustpower<2>(int x)            { return x*x; }" << endl;
+        
+        if (gFloatSize==1) {
+            
+            fout << "template <int N> inline float faustpower(float x)            { return faustpower<N/2>(x) * faustpower<N-N/2>(x); } " << endl;
+            fout << "template <> 	 inline float faustpower<0>(float x)          { return 1; }" << endl;
+            fout << "template <> 	 inline float faustpower<1>(float x)          { return x; }" << endl;
+            fout << "template <> 	 inline float faustpower<2>(float x)          { return x*x; }" << endl;
+            
+        } else if (gFloatSize==2) {
+        
+            fout << "template <int N> inline double faustpower(double x)          { return faustpower<N/2>(x) * faustpower<N-N/2>(x); } " << endl;
+            fout << "template <> 	 inline double faustpower<0>(double x)        { return 1; }" << endl;
+            fout << "template <> 	 inline double faustpower<1>(double x)        { return x; }" << endl;
+            fout << "template <> 	 inline double faustpower<2>(double x)        { return x*x; }" << endl;
+            
+        } else if (gFloatSize==3) {
+            
+            fout << "template <int N> inline long double faustpower(long double x){ return powl(x,N); }"  << endl;
+            
+        }
         fout << "#endif" << endl;
+
     }
 
 }
@@ -238,7 +261,7 @@ void Klass::printAdditionalCode(ostream& fout)
  */
 void Klass::printMetadata(int n, const map<Tree, set<Tree> >& S, ostream& fout)
 {
-    tab(n,fout); fout   << "static void metadata(Meta* m) \t{ ";
+    tab(n,fout); fout << "virtual void metadata(Meta* m) { ";
     
     // We do not want to accumulate metadata from all hierachical levels, so the upper level only is kept
     for (map<Tree, set<Tree> >::iterator i = gMetaDataSet.begin(); i != gMetaDataSet.end(); i++) {
@@ -301,25 +324,28 @@ static void computeUseCount(Loop* l)
 /**
  * Group together sequences of loops
  */
-static void groupSeqLoops(Loop* l)
+static void groupSeqLoops(Loop* l, set<Loop*>& visited)
 {
-	int n = (int)l->fBackwardLoopDependencies.size();
-	if (n==0) {
-		return;
-	} else if (n==1) {
-		Loop* f = *(l->fBackwardLoopDependencies.begin());
-		if (f->fUseCount ==  1) {
-			l->concat(f);
-			groupSeqLoops(l);
-		} else {
-			groupSeqLoops(f);
-		}
-		return;
-	} else if (n > 1) {
-		for (lset::iterator p =l->fBackwardLoopDependencies.begin(); p!=l->fBackwardLoopDependencies.end(); p++) {
-			groupSeqLoops(*p);
-		}
-	}
+    if (visited.find(l) == visited.end()) {
+        visited.insert(l);
+        int n = (int)l->fBackwardLoopDependencies.size();
+        if (n==0) {
+            return;
+        } else if (n==1) {
+            Loop* f = *(l->fBackwardLoopDependencies.begin());
+            if (f->fUseCount ==  1) {
+                l->concat(f);
+                groupSeqLoops(l, visited);
+            } else {
+                groupSeqLoops(f, visited);
+            }
+            return;
+        } else if (n > 1) {
+            for (lset::iterator p =l->fBackwardLoopDependencies.begin(); p!=l->fBackwardLoopDependencies.end(); p++) {
+                groupSeqLoops(*p, visited);
+            }
+        }
+    }
 }
 
 #define WORK_STEALING_INDEX 0
@@ -334,7 +360,8 @@ void Klass::buildTasksList()
 
     if (gGroupTaskSwitch) {
         computeUseCount(fTopLoop);
-        groupSeqLoops(fTopLoop);
+        set<Loop*> visited;
+        groupSeqLoops(fTopLoop, visited);
     }
 
     sortGraph(fTopLoop, G);
@@ -439,7 +466,6 @@ void Klass::buildTasksList()
 
     addInitCode("fStaticNumThreads = get_max_cpu();");
     addInitCode("fDynamicNumThreads = getenv(\"OMP_NUM_THREADS\") ? atoi(getenv(\"OMP_NUM_THREADS\")) : fStaticNumThreads;");
-    addInitCode("fThreadPool = DSPThreadPool::Init();");
     addInitCode("fThreadPool->StartAll(fStaticNumThreads - 1, false);");
 
     gTaskCount = 0;
@@ -452,7 +478,8 @@ void Klass::printLoopGraphVector(int n, ostream& fout)
 {
     if (gGroupTaskSwitch) {
         computeUseCount(fTopLoop);
-        groupSeqLoops(fTopLoop);
+        set<Loop*> visited;
+        groupSeqLoops(fTopLoop, visited);
     }
 
     lgraph G;
@@ -483,7 +510,8 @@ void Klass::printLoopGraphOpenMP(int n, ostream& fout)
 {
     if (gGroupTaskSwitch) {
         computeUseCount(fTopLoop);
-        groupSeqLoops(fTopLoop);
+        set<Loop*> visited;
+        groupSeqLoops(fTopLoop, visited);
     }
 
     lgraph G;
@@ -503,7 +531,8 @@ void Klass::printLoopGraphScheduler(int n, ostream& fout)
 {
     if (gGroupTaskSwitch) {
         computeUseCount(fTopLoop);
-        groupSeqLoops(fTopLoop);
+        set<Loop*> visited;
+        groupSeqLoops(fTopLoop, visited);
     }
 
     lgraph G;
@@ -760,21 +789,26 @@ void Klass::println(int n, ostream& fout)
     for (k = fSubClassList.begin(); k != fSubClassList.end(); k++) 	(*k)->println(n+1, fout);
 
     printlines(n+1, fDeclCode, fout);
+    
+    tab(n+1,fout); fout << "int fSamplingFreq;\n";
 
-	tab(n,fout); fout << "  public:";
+    tab(n,fout); fout << "  public:";
 
     printMetadata(n+1, gMetaDataSet, fout);
 
     if (gSchedulerSwitch) {
-        tab(n+1,fout); fout << "virtual ~" << fKlassName << "() \t{ "
-                            << "DSPThreadPool::Destroy()"
-                            << "; }";
+        tab(n+1,fout); fout << fKlassName << "() { "
+                            << "fThreadPool = DSPThreadPool::Init(); }";
+        
+        tab(n+1,fout); fout << "virtual ~" << fKlassName << "() { "
+                            << "DSPThreadPool::Destroy(); }";
     }
-
-    tab(n+1,fout); fout     << "virtual int getNumInputs() \t{ "
+    
+    tab(n+1,fout); fout << "virtual int getNumInputs() { "
                     << "return " << fNumInputs
                     << "; }";
-    tab(n+1,fout); fout 	<< "virtual int getNumOutputs() \t{ "
+    
+    tab(n+1,fout); fout << "virtual int getNumOutputs() { "
                     << "return " << fNumOutputs
                     << "; }";
 
@@ -782,18 +816,39 @@ void Klass::println(int n, ostream& fout)
         printlines (n+2, fStaticInitCode, fout);
     tab(n+1,fout); fout << "}";
 
-    tab(n+1,fout); fout << "virtual void instanceInit(int samplingFreq) {";
+    tab(n+1,fout); fout << "virtual void instanceConstants(int samplingFreq) {";
         tab(n+2,fout); fout << "fSamplingFreq = samplingFreq;";
         printlines (n+2, fInitCode, fout);
     tab(n+1,fout); fout << "}";
+    
+    tab(n+1,fout); fout << "virtual void instanceResetUserInterface() {";
+        printlines (n+2, fInitUICode, fout);
+    tab(n+1,fout); fout << "}";
+    
+    tab(n+1,fout); fout << "virtual void instanceClear() {";
+        printlines (n+2, fClearCode, fout);
+    tab(n+1,fout); fout << "}";
 
     tab(n+1,fout); fout << "virtual void init(int samplingFreq) {";
         tab(n+2,fout); fout << "classInit(samplingFreq);";
-         tab(n+2,fout); fout << "instanceInit(samplingFreq);";
+        tab(n+2,fout); fout << "instanceInit(samplingFreq);";
+    tab(n+1,fout); fout << "}";
+    
+    tab(n+1,fout); fout << "virtual void instanceInit(int samplingFreq) {";
+    tab(n+2,fout); fout << "instanceConstants(samplingFreq);";
+    tab(n+2,fout); fout << "instanceResetUserInterface();";
+    tab(n+2,fout); fout << "instanceClear();";
+    tab(n+1,fout); fout << "}";
+    
+    tab(n+1,fout); fout << "virtual "<< fKlassName <<"* clone() {";
+        tab(n+2,fout); fout << "return new " << fKlassName << "();";
+    tab(n+1,fout); fout << "}";
+    
+    tab(n+1,fout); fout << "virtual int getSampleRate() {";
+        tab(n+2,fout); fout << "return fSamplingFreq;";
     tab(n+1,fout); fout << "}";
 
-
-    tab(n+1,fout); fout << "virtual void buildUserInterface(UI* interface) {";
+    tab(n+1,fout); fout << "virtual void buildUserInterface(UI* ui_interface) {";
         printlines (n+2, fUICode, fout);
     tab(n+1,fout); fout << "}";
 
@@ -1144,7 +1199,7 @@ void SigIntGenKlass::println(int n, ostream& fout)
 	tab(n,fout); fout << "class " << fKlassName << " {";
 
 	tab(n,fout); fout << "  private:";
-		tab(n+1,fout); fout << "int \tfSamplingFreq;";
+		tab(n+1,fout); fout << "int fSamplingFreq;";
 
 		for (k = fSubClassList.begin(); k != fSubClassList.end(); k++) 	(*k)->println(n+1, fout);
 
@@ -1152,14 +1207,15 @@ void SigIntGenKlass::println(int n, ostream& fout)
 
 	tab(n,fout); fout << "  public:";
 
-		tab(n+1,fout); fout 	<< "int getNumInputs() \t{ "
+		tab(n+1,fout); fout 	<< "int getNumInputs() { "
 						<< "return " << fNumInputs << "; }";
-		tab(n+1,fout); fout 	<< "int getNumOutputs() \t{ "
+		tab(n+1,fout); fout 	<< "int getNumOutputs() { "
 						<< "return " << fNumOutputs << "; }";
 
 		tab(n+1,fout); fout << "void init(int samplingFreq) {";
 			tab(n+2,fout); fout << "fSamplingFreq = samplingFreq;";
             printlines(n+2, fInitCode, fout);
+            printlines(n+2, fClearCode, fout);
 		tab(n+1,fout); fout << "}";
 
 		tab(n+1,fout); fout << "void fill (int count, int output[]) {";
@@ -1183,7 +1239,7 @@ void SigFloatGenKlass::println(int n, ostream& fout)
 	tab(n,fout); fout << "class " << fKlassName << " {";
 
 	tab(n,fout); fout << "  private:";
-		tab(n+1,fout); fout << "int \tfSamplingFreq;";
+		tab(n+1,fout); fout << "int fSamplingFreq;";
 
 		for (k = fSubClassList.begin(); k != fSubClassList.end(); k++) 	(*k)->println(n+1, fout);
 
@@ -1191,14 +1247,15 @@ void SigFloatGenKlass::println(int n, ostream& fout)
 
 	tab(n,fout); fout << "  public:";
 
-		tab(n+1,fout); fout 	<< "int getNumInputs() \t{ "
+		tab(n+1,fout); fout 	<< "int getNumInputs() { "
 						<< "return " << fNumInputs << "; }";
-		tab(n+1,fout); fout 	<< "int getNumOutputs() \t{ "
+		tab(n+1,fout); fout 	<< "int getNumOutputs() { "
 						<< "return " << fNumOutputs << "; }";
 
 		tab(n+1,fout); fout << "void init(int samplingFreq) {";
 			tab(n+2,fout); fout << "fSamplingFreq = samplingFreq;";
 			printlines(n+2, fInitCode, fout);
+            printlines(n+2, fClearCode, fout);
 		tab(n+1,fout); fout << "}";
 
 		tab(n+1,fout); fout << subst("void fill (int count, $0 output[]) {", ifloat());
