@@ -1,7 +1,7 @@
 /************************************************************************
  ************************************************************************
     FAUST compiler
-	Copyright (C) 2003-2012 GRAME, Centre National de Creation Musicale
+	Copyright (C) 2003-2016 GRAME, Centre National de Creation Musicale
     ---------------------------------------------------------------------
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -18,7 +18,7 @@
     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  ************************************************************************
  ************************************************************************/
-#define FAUSTVERSION "0.9.73"
+#define FAUSTVERSION "0.9.95"
 
 #include <stdlib.h>
 #include <stdio.h>
@@ -194,10 +194,8 @@ static string makeDrawPathNoExt()
 {
     if (gOutputDir != "") {
         return gOutputDir + "/" + gMasterName;
-    } else if (gMasterDocument.length() >= 4 && gMasterDocument.substr(gMasterDocument.length() - 4) == ".dsp") {
-        return gMasterDocument.substr(0, gMasterDocument.length() - 4);
     } else {
-        return gMasterDocument;
+        return strip_end(gMasterDocument, ".dsp");
     }
 }
 
@@ -210,38 +208,38 @@ static string makeDrawPathNoExt()
 
 bool process_cmdline(int argc, char* argv[])
 {
-	int	i=1; int err=0;
+    int	i=1; int err=0;
 
-	while (i<argc) {
-   
-		if (isCmd(argv[i], "-h", "--help")) {
-			gHelpSwitch = true;
-			i += 1;
+    while (i<argc) {
 
-		} else if (isCmd(argv[i], "-v", "--version")) {
-			gVersionSwitch = true;
-			i += 1;
+        if (isCmd(argv[i], "-h", "--help")) {
+            gHelpSwitch = true;
+            i += 1;
 
-		} else if (isCmd(argv[i], "-d", "--details")) {
-			gDetailsSwitch = true;
-			i += 1;
+        } else if (isCmd(argv[i], "-v", "--version")) {
+            gVersionSwitch = true;
+            i += 1;
 
-        } else if (isCmd(argv[i], "-a", "--architecture")) {
+        } else if (isCmd(argv[i], "-d", "--details")) {
+            gDetailsSwitch = true;
+            i += 1;
+
+        } else if (isCmd(argv[i], "-a", "--architecture") && (i+1 < argc)) {
             gArchFile = argv[i+1];
             i += 2;
 
-        } else if (isCmd(argv[i], "-inj", "--inject")) {
+        } else if (isCmd(argv[i], "-inj", "--inject") && (i+1 < argc)) {
             gInjectFlag = true;
             gInjectFile = argv[i+1];
             i += 2;
 
-		} else if (isCmd(argv[i], "-o")) {
-			gOutputFile = argv[i+1];
-			i += 2;
+        } else if (isCmd(argv[i], "-o") && (i+1 < argc)) {
+            gOutputFile = argv[i+1];
+            i += 2;
 
-		} else if (isCmd(argv[i], "-ps", "--postscript")) {
-			gDrawPSSwitch = true;
-			i += 1;
+        } else if (isCmd(argv[i], "-ps", "--postscript")) {
+            gDrawPSSwitch = true;
+            i += 1;
 
         } else if (isCmd(argv[i], "-xml", "--xml")) {
             gPrintXMLSwitch = true;
@@ -267,45 +265,45 @@ bool process_cmdline(int argc, char* argv[])
             gScaledSVG = true;
             i += 1;
 
-		} else if (isCmd(argv[i], "-svg", "--svg")) {
-			gDrawSVGSwitch = true;
-			i += 1;
+        } else if (isCmd(argv[i], "-svg", "--svg")) {
+            gDrawSVGSwitch = true;
+            i += 1;
 
-		} else if (isCmd(argv[i], "-f", "--fold")) {
-			gFoldThreshold = atoi(argv[i+1]);
-			i += 2;
+        } else if (isCmd(argv[i], "-f", "--fold") && (i+1 < argc)) {
+            gFoldThreshold = atoi(argv[i+1]);
+            i += 2;
 
-		} else if (isCmd(argv[i], "-mns", "--max-name-size")) {
-			gMaxNameSize = atoi(argv[i+1]);
-			i += 2;
+        } else if (isCmd(argv[i], "-mns", "--max-name-size") && (i+1 < argc)) {
+            gMaxNameSize = atoi(argv[i+1]);
+            i += 2;
 
-		} else if (isCmd(argv[i], "-sn", "--simple-names")) {
-			gSimpleNames = true;
-			i += 1;
+        } else if (isCmd(argv[i], "-sn", "--simple-names")) {
+            gSimpleNames = true;
+            i += 1;
 
-		} else if (isCmd(argv[i], "-lb", "--left-balanced")) {
-			gBalancedSwitch = 0;
-			i += 1;
+        } else if (isCmd(argv[i], "-lb", "--left-balanced")) {
+            gBalancedSwitch = 0;
+            i += 1;
 
-		} else if (isCmd(argv[i], "-mb", "--mid-balanced")) {
-			gBalancedSwitch = 1;
-			i += 1;
+        } else if (isCmd(argv[i], "-mb", "--mid-balanced")) {
+            gBalancedSwitch = 1;
+            i += 1;
 
-		} else if (isCmd(argv[i], "-rb", "--right-balanced")) {
-			gBalancedSwitch = 2;
-			i += 1;
+        } else if (isCmd(argv[i], "-rb", "--right-balanced")) {
+            gBalancedSwitch = 2;
+            i += 1;
 
-		} else if (isCmd(argv[i], "-lt", "--less-temporaries")) {
-			gLessTempSwitch = true;
-			i += 1;
+        } else if (isCmd(argv[i], "-lt", "--less-temporaries")) {
+            gLessTempSwitch = true;
+            i += 1;
 
-		} else if (isCmd(argv[i], "-mcd", "--max-copy-delay")) {
-			gMaxCopyDelay = atoi(argv[i+1]);
-			i += 2;
+        } else if (isCmd(argv[i], "-mcd", "--max-copy-delay") && (i+1 < argc)) {
+            gMaxCopyDelay = atoi(argv[i+1]);
+            i += 2;
 
-		} else if (isCmd(argv[i], "-sd", "--simplify-diagrams")) {
-			gSimplifyDiagrams = true;
-			i += 1;
+        } else if (isCmd(argv[i], "-sd", "--simplify-diagrams")) {
+            gSimplifyDiagrams = true;
+            i += 1;
 
         } else if (isCmd(argv[i], "-vec", "--vectorize")) {
             gVectorSwitch = true;
@@ -315,11 +313,11 @@ bool process_cmdline(int argc, char* argv[])
             gDeepFirstSwitch = true;
             i += 1;
 
-        } else if (isCmd(argv[i], "-vs", "--vec-size")) {
+        } else if (isCmd(argv[i], "-vs", "--vec-size") && (i+1 < argc)) {
             gVecSize = atoi(argv[i+1]);
             i += 2;
 
-        } else if (isCmd(argv[i], "-lv", "--loop-variant")) {
+        } else if (isCmd(argv[i], "-lv", "--loop-variant") && (i+1 < argc)) {
             gVectorLoopVariant = atoi(argv[i+1]);
             i += 2;
 
@@ -332,18 +330,18 @@ bool process_cmdline(int argc, char* argv[])
             i += 1;
 
         } else if (isCmd(argv[i], "-sch", "--scheduler")) {
-			gSchedulerSwitch = true;
-			i += 1;
+            gSchedulerSwitch = true;
+            i += 1;
 
         } else if (isCmd(argv[i], "-g", "--groupTasks")) {
-			gGroupTaskSwitch = true;
-			i += 1;
+            gGroupTaskSwitch = true;
+            i += 1;
 
         } else if (isCmd(argv[i], "-uim", "--user-interface-macros")) {
-			gUIMacroSwitch = true;
-			i += 1;
+            gUIMacroSwitch = true;
+            i += 1;
 
-        } else if (isCmd(argv[i], "-t", "--timeout")) {
+        } else if (isCmd(argv[i], "-t", "--timeout") && (i+1 < argc)) {
             gTimeout = atoi(argv[i+1]);
             i += 2;
             
@@ -363,19 +361,19 @@ bool process_cmdline(int argc, char* argv[])
         } else if (isCmd(argv[i], "-quad", "--quad-precision-floats")) {
             gFloatSize = 3;
             i += 1;
-			
+            
         } else if (isCmd(argv[i], "-mdoc", "--mathdoc")) {
             gPrintDocSwitch = true;
             i += 1;
-			
-        } else if (isCmd(argv[i], "-mdlang", "--mathdoc-lang")) {
+            
+        } else if (isCmd(argv[i], "-mdlang", "--mathdoc-lang") && (i+1 < argc)) {
             gDocLang = argv[i+1];
             i += 2;
-			
+            
         } else if (isCmd(argv[i], "-stripmdoc", "--strip-mdoc-tags")) {
             gStripDocSwitch = true;
             i += 1;
-			
+            
         } else if (isCmd(argv[i], "-flist", "--file-list")) {
             gPrintFileListSwitch = true;
             i += 1;
@@ -384,9 +382,9 @@ bool process_cmdline(int argc, char* argv[])
             gDumpNorm = true;
             i += 1;
 
-		} else if (isCmd(argv[i], "-cn", "--class-name")) {
-			gClassName = argv[i+1];
-			i += 2;
+        } else if (isCmd(argv[i], "-cn", "--class-name") && (i+1 < argc)) {
+            gClassName = argv[i+1];
+            i += 2;
 
         } else if (isCmd(argv[i], "-i", "--inline-architecture-files")) {
             gInlineArchSwitch = true;
@@ -396,24 +394,27 @@ bool process_cmdline(int argc, char* argv[])
             gExportDSP = true;
             i += 1;
 
-        } else if (isCmd(argv[i], "-I", "--import-dir")) {
-
+        } else if (isCmd(argv[i], "-I", "--import-dir") && (i+1 < argc)) {
+            if (strstr(argv[i+1], "http://") != 0) {
+                gImportDirList.push_back(argv[i+1]);
+            } else {
+                char temp[PATH_MAX+1];
+                char* path = realpath(argv[i+1], temp);
+                if (path == 0) {
+                    std::cerr << "ERROR : invalid directory path " << argv[i+1] << std::endl;
+                    exit(-1);
+                } else {
+                    gImportDirList.push_back(path);
+                    i += 2;
+                }
+            }
+            
+         } else if (isCmd(argv[i], "-O", "--output-dir") && (i+1 < argc)) {
             char temp[PATH_MAX+1];
             char* path = realpath(argv[i+1], temp);
             if (path == 0) {
                 std::cerr << "ERROR : invalid directory path " << argv[i+1] << std::endl;
                 exit(-1);
-            } else {
-                gImportDirList.push_back(path);
-                i += 2;
-            }
-         } else if (isCmd(argv[i], "-O", "--output-dir")) {
-        
-            char temp[PATH_MAX+1];
-            char* path = realpath(argv[i+1], temp);
-             if (path == 0) {
-                std::cerr << "ERROR : invalid directory path " << argv[i+1] << std::endl;
-                exit(-1);
             } else {
                 gOutputDir = path;
                 i += 2;
@@ -428,24 +429,25 @@ bool process_cmdline(int argc, char* argv[])
             if (check_url(url)) {
                 gInputFiles.push_back(url);
             }
-			i++;
+            i++;
 
-		} else {
-			std::cerr << "faust: unrecognized option \"" << argv[i] <<"\"" << endl;
-			i++;
-			err++;
-		}
-	}
+        } else {
+            std::cerr << "ERROR : unrecognized or incorrectly defined option \"" << argv[i] <<"\"" << endl;
+            i++;
+            err++;
+            exit(-1);
+        }
+    }
 
     // adjust related options
     if (gOpenMPSwitch || gSchedulerSwitch) gVectorSwitch = true;
-    
+
     if (gInPlace && gVectorSwitch) {
         std::cerr << "ERROR : 'in-place' option can only be used in scalar mode" << endl;
         exit(-1);
     }   
 
-	return err == 0;
+    return err == 0;
 }
 
 
@@ -459,7 +461,7 @@ bool process_cmdline(int argc, char* argv[])
 void printversion()
 {
 	cout << "FAUST, DSP to C++ compiler, Version " << FAUSTVERSION << "\n";
-    cout << "Copyright (C) 2002-2014, GRAME - Centre National de Creation Musicale. All rights reserved. \n\n";
+    cout << "Copyright (C) 2002-2016, GRAME - Centre National de Creation Musicale. All rights reserved. \n\n";
 }
 
 
@@ -481,7 +483,7 @@ void printhelp()
     cout << "-svg \t\tprint block-diagram --svg file\n";
     cout << "-mdoc \t\tprint --mathdoc of a Faust program in LaTeX format in a -mdoc directory\n";
     cout << "-mdlang <l> \tload --mathdoc-lang <l> if translation file exists (<l> = en, fr, ...)\n";
-    cout << "-stripdoc  \tapply --strip-mdoc-tags when printing Faust -mdoc listings\n";
+    cout << "-stripmdoc  \tapply --strip-mdoc-tags when printing Faust -mdoc listings\n";
     cout << "-sd \t\ttry to further --simplify-diagrams before drawing them\n";
 	cout << "-f <n> \t\t--fold <n> threshold during block-diagram generation (default 25 elements) \n";
 	cout << "-mns <n> \t--max-name-size <n> threshold during block-diagram generation (default 40 char)\n";
@@ -536,7 +538,7 @@ void printheader(ostream& dst)
     selectedKeys.insert(tree("license"));
     selectedKeys.insert(tree("version"));
 
-    dst << "//-----------------------------------------------------" << endl;
+    dst << "//----------------------------------------------------------" << endl;
     for (map<Tree, set<Tree> >::iterator i = gMetaDataSet.begin(); i != gMetaDataSet.end(); i++) {
         if (selectedKeys.count(i->first)) {
             dst << "// " << *(i->first);
@@ -551,7 +553,7 @@ void printheader(ostream& dst)
 
     dst << "//" << endl;
     dst << "// Code generated with Faust " << FAUSTVERSION << " (http://faust.grame.fr)" << endl;
-    dst << "//-----------------------------------------------------" << endl;
+    dst << "//----------------------------------------------------------" << endl << endl;
 }
 
 
@@ -611,8 +613,8 @@ static void initFaustDirectories()
 int main (int argc, char* argv[])
 {
     ostream*    dst;
-    ifstream*   injcode;
-    istream*    enrobage;
+    ifstream*   injcode=0;
+    istream*    enrobage=0;
 
 
 	/****************************************************************
@@ -782,9 +784,8 @@ int main (int argc, char* argv[])
 	else if (gVectorSwitch) C = new VectorCompiler(gClassName, "dsp", numInputs, numOutputs);
 	else                    C = new ScalarCompiler(gClassName, "dsp", numInputs, numOutputs);
 
-	if (gPrintXMLSwitch) C->setDescription(new Description());
-	if (gPrintDocSwitch) C->setDescription(new Description());
-
+	if (gPrintXMLSwitch || gPrintDocSwitch) C->setDescription(new Description());
+	
 	C->compileMultiSignal(lsignals);
 
 	endTiming("compilation");
