@@ -538,23 +538,25 @@ Automaton *make_pattern_matcher(Tree R)
     if (A->final(s)) {
       list<Rule>::const_iterator ru;
       for (ru = A->rules(s).begin(); ru != A->rules(s).end(); ru++)
-	if (!isBoxError(E[ru->r]))
-	  if (ru->r < r) {
-	    /* Lhs of rule #r matched a higher-priority rule, so rule #r may
-	       be shadowed. */
-	    Tree lhs1, rhs1, lhs2, rhs2;
-	    if (isCons(rules[ru->r], lhs1, rhs1) &&  isCons(rules[r], lhs2, rhs2)) {
-			cerr 	<< "WARNING : shadowed pattern-matching rule: "
-				<< boxpp(reverse(lhs2)) << " => " << boxpp(rhs2) << ";"
-				<< " previous rule was: " 
-				<< boxpp(reverse(lhs1)) << " => " << boxpp(rhs1) << ";"
-				<< endl;
-		} else {
-			cerr << "INTERNAL ERROR : " << __FILE__ << ":" << __LINE__ << endl;
-			exit(1);
-		}
-	  } else if (ru->r >= r)
-	    break;
+      if (!isBoxError(E[ru->r])) {
+          if (ru->r < r) {
+            /* Lhs of rule #r matched a higher-priority rule, so rule #r may
+               be shadowed. */
+            Tree lhs1, rhs1, lhs2, rhs2;
+            if (isCons(rules[ru->r], lhs1, rhs1) &&  isCons(rules[r], lhs2, rhs2)) {
+                cerr 	<< "WARNING : shadowed pattern-matching rule: "
+                    << boxpp(reverse(lhs2)) << " => " << boxpp(rhs2) << ";"
+                    << " previous rule was: " 
+                    << boxpp(reverse(lhs1)) << " => " << boxpp(rhs1) << ";"
+                    << endl;
+            } else {
+                cerr << "INTERNAL ERROR : " << __FILE__ << ":" << __LINE__ << endl;
+                exit(1);
+            }
+          } else if (ru->r >= r) {
+            break;
+          }
+       }
     }
   }
 #ifdef DEBUG
